import smbus
import time

# MPU6050 Registers and Addresses
DEVICE_ADDRESS = 0x68       # MPU6050 device address

PWR_MGMT_1 = 0x6B
SMPLRT_DIV = 0x19
CONFIG = 0x1A
GYRO_CONFIG = 0x1B
ACCEL_CONFIG = 0x1C
INT_ENABLE = 0x38

ACCEL_XOUT_H = 0x3B
ACCEL_YOUT_H = 0x3D
ACCEL_ZOUT_H = 0x3F

GYRO_XOUT_H = 0x43
GYRO_YOUT_H = 0x45
GYRO_ZOUT_H = 0x47

def calibrate_gyroscope(samples=1000):
    print("Calibrating gyroscope... Please keep the sensor stationary.")
    gyro_x_offset = 0
    gyro_y_offset = 0
    gyro_z_offset = 0

    for _ in range(samples):
        gyro_x = read_raw_data(GYRO_XOUT_H)
        gyro_y = read_raw_data(GYRO_YOUT_H)
        gyro_z = read_raw_data(GYRO_ZOUT_H)

        gyro_x_offset += gyro_x
        gyro_y_offset += gyro_y
        gyro_z_offset += gyro_z

        time.sleep(0.00001)  # Adjust the delay as per your sampling rate

    gyro_x_offset /= samples
    gyro_y_offset /= samples
    gyro_z_offset /= samples

    print("Gyroscope calibration complete.")
    return gyro_x_offset, gyro_y_offset, gyro_z_offset

def calibrate_accelerometer(samples=1000):
    print("Calibrating gyroscope... Please keep the sensor stationary.")
    acc_x_offset = 0
    acc_y_offset = 0
    acc_z_offset = 0

    for _ in range(samples):
        acc_x = read_raw_data(ACCEL_XOUT_H)
        acc_y = read_raw_data(ACCEL_YOUT_H)
        acc_z = read_raw_data(ACCEL_ZOUT_H)

        acc_x_offset += acc_x
        acc_y_offset += acc_y
        acc_z_offset += acc_z

        time.sleep(0.00001)  # Adjust the delay as per your sampling rate

    acc_x_offset /= samples
    acc_y_offset /= samples
    acc_z_offset /= samples
    

    print("Gyroscope calibration complete.")
    return acc_x_offset, acc_y_offset, acc_z_offset





def MPU_Init():
    # Write to sample rate register
    bus.write_byte_data(DEVICE_ADDRESS, SMPLRT_DIV, 7)
    # Write to power management register to wake up the MPU6050
    bus.write_byte_data(DEVICE_ADDRESS, PWR_MGMT_1, 1)
    # Write to configuration register
    bus.write_byte_data(DEVICE_ADDRESS, CONFIG, 0)
    # Write to gyroscope configuration register
    bus.write_byte_data(DEVICE_ADDRESS, GYRO_CONFIG, 24)
    # Write to accelerometer configuration register
    bus.write_byte_data(DEVICE_ADDRESS, ACCEL_CONFIG, 0)
    # Enable interrupts
    bus.write_byte_data(DEVICE_ADDRESS, INT_ENABLE, 1)
    
    
    

def read_raw_data(addr):
    # Read two bytes of data from the given address
    high = bus.read_byte_data(DEVICE_ADDRESS, addr)
    low = bus.read_byte_data(DEVICE_ADDRESS, addr+1)
    # Combine higher and lower bytes
    value = (high << 8) | low
    # Convert to signed integer
    if value > 32768:
        value = value - 65536
    return value



# Initialize the I2C bus
bus = smbus.SMBus(1)  # Use bus number 1 for Raspberry Pi

# Initialize MPU6050
MPU_Init()

print("Reading data from MPU6050...")

gyro_x_offset, gyro_y_offset, gyro_z_offset = calibrate_gyroscope()
acc_x_offset, acc_y_offset, acc_z_offset = calibrate_accelerometer()

try:
#     while True:
        # Read accelerometer data
        acc_x = read_raw_data(ACCEL_XOUT_H)
        acc_y = read_raw_data(ACCEL_YOUT_H)
        acc_z = read_raw_data(ACCEL_ZOUT_H)
        
        # Read gyroscope data
        gyro_x = read_raw_data(GYRO_XOUT_H)
        gyro_y = read_raw_data(GYRO_YOUT_H)
        gyro_z = read_raw_data(GYRO_ZOUT_H)
    
        
        # Convert raw data to meaningful units
        Ax = acc_x / 16384.0  # Accelerometer sensitivity scale factor
        Ay = acc_y / 16384.0
        Az = acc_z / 16384.0
        
        Gx = gyro_x / 131.0   # Gyroscope sensitivity scale factor
        Gy = gyro_y / 131.0
        Gz = gyro_z / 131.0
  
          
        Aox = acc_x_offset / 16384.0  # Accelerometer sensitivity scale factor
        Aoy = acc_y_offset / 16384.0
        Aoz = (acc_z_offset / 16384.0)-1
        
        
        Gox = gyro_x_offset / 131.0
        Goy = gyro_y_offset / 131.0
        Goz = gyro_z_offset / 131.0

        # Print the results
        #print(f"Accelerometer (g): X={Ax:.8f}, Y={Ay:.8f}, Z={Az:.8f}")
        #print(f"Gyroscope (°/s): X={Gx:.8f}, Y={Gy:.8f}, Z={Gz:.8f}")
        print("-" * 50)
        
        # Print the offset
        print(f"Accelerometer Offset (g): X={Aox:.8f}, Y={Aoy:.8f}, Z={Aoz:.8f}")
        print(f"Gyroscope Offset (°/s): X={Gox:.8f}, Y={Goy:.8f}, Z={Goz:.8f}")
        print("-" * 50)
        
        
        # Delay for readability
        # time.sleep(0.1)

except KeyboardInterrupt:
    print("\nTerminated by User")
      