import smbus
import time
import math

# MPU6050 Registers and Addresses
DEVICE_ADDRESS = 0x68       # MPU6050 device address

PWR_MGMT_1 = 0x6B
SMPLRT_DIV = 0x19
CONFIG = 0x1A
GYRO_CONFIG = 0x1B
ACCEL_CONFIG = 0x1C
INT_ENABLE = 0x38

ACCEL_XOUT_H = 0x3B
ACCEL_YOUT_H = 0x3D
ACCEL_ZOUT_H = 0x3F

GYRO_XOUT_H = 0x43
GYRO_YOUT_H = 0x45
GYRO_ZOUT_H = 0x47

def MPU_Init():
    # Write to sample rate register
    bus.write_byte_data(DEVICE_ADDRESS, SMPLRT_DIV, 7)
    # Write to power management register to wake up the MPU6050
    bus.write_byte_data(DEVICE_ADDRESS, PWR_MGMT_1, 1)
    # Write to configuration register
    bus.write_byte_data(DEVICE_ADDRESS, CONFIG, 0)
    # Write to gyroscope configuration register
    bus.write_byte_data(DEVICE_ADDRESS, GYRO_CONFIG, 24)
    # Write to accelerometer configuration register
    bus.write_byte_data(DEVICE_ADDRESS, ACCEL_CONFIG, 0)
    # Enable interrupts
    bus.write_byte_data(DEVICE_ADDRESS, INT_ENABLE, 1)

def read_raw_data(addr):
    # Read two bytes of data from the given address
    high = bus.read_byte_data(DEVICE_ADDRESS, addr)
    low = bus.read_byte_data(DEVICE_ADDRESS, addr+1)
    # Combine higher and lower bytes
    value = (high << 8) | low
    # Convert to signed integer
    if value > 32768:
        value = value - 65536
    return value

# Initialize the I2C bus
bus = smbus.SMBus(1)  # Use bus number 1 for Raspberry Pi

# Initialize MPU6050
MPU_Init()

print("Reading data from MPU6050...")

try:
    while True:
        # Read accelerometer data
        acc_x = read_raw_data(ACCEL_XOUT_H)
        acc_y = read_raw_data(ACCEL_YOUT_H)
        acc_z = read_raw_data(ACCEL_ZOUT_H)
        
        # Read gyroscope data
        gyro_x = read_raw_data(GYRO_XOUT_H)
        gyro_y = read_raw_data(GYRO_YOUT_H)
        gyro_z = read_raw_data(GYRO_ZOUT_H)
        
        # Convert raw data to meaningful units
        Ax = (acc_x / 16384.0 - 0.01467432) # Accelerometer sensitivity scale factor
        Ay = (acc_y / 16384.0 - -0.00603101)
        Az = (acc_z / 16384.0 - 0.06171924)
        
        Ax_m = 9.81*Ax
        Ay_m = 9.81*Ay
        Az_m = 9.81*Az
        
        Gx = gyro_x / 131.0  --0.17447328# Gyroscope sensitivity scale factor
        Gy = gyro_y / 131.0  -0.08720611
        Gz = gyro_z / 131.0  -0.16423664
        
        Gx_r = Gx/180 *math.pi
        Gy_r = Gy/180 *math.pi
        Gz_r = Gz/180 *math.pi
        
        # Print the results
        print(f"Accelerometer (g): X={Ax:.3f}, Y={Ay:.3f}, Z={Az:.3f}")
        print(f"Gyroscope (°/s): X={Gx:.3f}, Y={Gy:.3f}, Z={Gz:.3f}")
        
        print(f"Accelerometer (m/s^2): X={Ax_m:.3f}, Y={Ay_m:.3f}, Z={Az_m:.3f}")
        print(f"Gyroscope (rad/s): X={Gx_r:.3f}, Y={Gy_r:.3f}, Z={Gz_r:.3f}")
        print("-" * 50)
        
        # Delay for readability
        time.sleep(0.1)

except KeyboardInterrupt:
    print("\nTerminated by User")
      

