import pigpio
from gpiozero import PWMOutputDevice
from time import sleep, time
import threading

# Motor control class
class Motor:
    def __init__(self, EnaA, In1A, In2A, EnaB, In1B, In2B):
        # Initialize PWM and GPIO pins for both motors
        self.pwmA = PWMOutputDevice(EnaA)
        self.in1A = PWMOutputDevice(In1A)
        self.in2A = PWMOutputDevice(In2A)
        self.pwmB = PWMOutputDevice(EnaB)
        self.in1B = PWMOutputDevice(In1B)
        self.in2B = PWMOutputDevice(In2B)
        
    def move(self, speed=0.5, turn=0, t=0):
        # Ensure speed and turn are within bounds
        speed = max(-1, min(1, speed))  # Clamp speed to [-1, 1]
        turn = max(-1, min(1, turn))    # Clamp turn to [-1, 1]
        
        # Calculate speed for each motor
        leftSpeed = speed - turn
        rightSpeed = speed + turn
         
        # Clamp motor speeds to [-1, 1]
        leftSpeed = max(-1, min(1, leftSpeed))
        rightSpeed = max(-1, min(1, rightSpeed))
        
        # Set left motor
        self.pwmA.value = abs(leftSpeed)
        self.in1A.value = leftSpeed > 0
        self.in2A.value = leftSpeed <= 0
        
        # Set right motor
        self.pwmB.value = abs(rightSpeed)
        self.in1B.value = rightSpeed > 0
        self.in2B.value = rightSpeed <= 0
        
        # Print debugging info
        print(f"Left Motor: Speed={leftSpeed}, Right Motor: Speed={rightSpeed}")
        
        sleep(t)
        
    def stop(self, t=0):
        self.pwmA.value = 0
        self.pwmB.value = 0
        sleep(t)

# Encoder class using pigpio
class Encoder:
    def __init__(self, gpioA, gpioB):
        self.pi = pigpio.pi()
        if not self.pi.connected:
            raise IOError("Could not connect to pigpio daemon!")
        self.gpioA = gpioA
        self.gpioB = gpioB
        self.position = 0

        self.pi.set_mode(gpioA, pigpio.INPUT)
        self.pi.set_mode(gpioB, pigpio.INPUT)

        self.pi.set_pull_up_down(gpioA, pigpio.PUD_UP)
        self.pi.set_pull_up_down(gpioB, pigpio.PUD_UP)

        self.lastGpio = None

        self.callbackA = self.pi.callback(gpioA, pigpio.EITHER_EDGE, self._pulse)
        self.callbackB = self.pi.callback(gpioB, pigpio.EITHER_EDGE, self._pulse)

    def _pulse(self, gpio, level, tick):
        if gpio == self.gpioA:
            levelB = self.pi.read(self.gpioB)
            if level == levelB:
                self.position += 1
            else:
                self.position -= 1
        else:
            levelA = self.pi.read(self.gpioA)
            if level == levelA:
                self.position -= 1
            else:
                self.position += 1

    def get_position(self):
        return self.position

    def cancel(self):
        self.callbackA.cancel()
        self.callbackB.cancel()
        self.pi.stop()

def main():
    # Initialize motor and encoder
    motor = Motor(2, 3, 4, 17, 22, 27)
    encoder1 = Encoder(5, 6)
    encoder2 = Encoder(13, 19)
    
    try:
        # Start motor movement in a separate thread
        def move_motor():
            print("Starting motor...")
            motor.move(0.5, 0)  # Start moving forward
            sleep(3)            # Move for 3 seconds
            motor.stop()
            print("Motor stopped.")

        motor_thread = threading.Thread(target=move_motor)
        motor_thread.start()

        # Read encoder values while motor is running
        start_time = time()
        while motor_thread.is_alive():
            pos1 = encoder1.get_position()
            pos2 = encoder2.get_position()
            elapsed_time = time() - start_time
            print(f"Time: {elapsed_time:.2f}s, Motor 1 Position: {pos1}, Motor 2 Position: {pos2}")
            sleep(0.1)  # Adjust the sleep time as needed

        # Final encoder positions
        pos1 = encoder1.get_position()
        pos2 = encoder2.get_position()
        print(f"Final Positions - Motor 1: {pos1}, Motor 2: {pos2}")

    except KeyboardInterrupt:
        print("Program interrupted by user.")
    finally:
        # Clean up
        encoder1.cancel()
        encoder2.cancel()
        motor.stop()
        print("Program terminated.")

if __name__ == '__main__':
    main()
